<?php
/** 
 * @version $Id: ImageSizer.php 65 2007-01-20 01:50:30Z thepaper $
 *
 * ImageSizer.php - resize standard web image (gif/jpg/png)
 *
 * @author  Jack Pham (www.reversefolds.com)
 *          copyright (c) 2006 
 *
 * License and Terms:
 * This software is provided AS-IS with no explicit or implied warranties 
 * or garantees whatsoever.  USE AT OWN RISK.  The original author,
 * hosting provider and anyone/everyone he knows is NOT LIABLE for any
 * damages incurred using, downloading or looking at this this 
 * software.
 *
 * Please send any bugfixes/modifications to phamdev@yahoo.com
 *
 * ========================================================================
 * Usage:
 *
 * // create a thumbnail file in images/pics/thumbs/ dir 
 * // (dir created if it doesn't exist) with set width
 * $image = new ImageSizer('pic.gif', 'images/pics/', 75);  
 * $thumbnail = $image->getThumbnail();  // 'images/pics/thumbs/th_pic.gif'
 *
 * // set the height
 * $image = new ImageSizer('pic.gif', 'images/', 'auto', 150);
 *
 * // size by percentage (75%)
 * $image = new ImageSizer('pic.gif', 'images/', 'auto', 'auto', 0.75);
 *
 * // create a sub directory under thumbs
 * $image->setSubDir('mythumbs'); 
 *
 * // place thumbs in user-defined directory (instead of the default 'thumbs'
 * $image->setOutputDir('thumbdir');
 * ========================================================================
 */
class ImageSizer
{ 

    /** 
     * image filename including path 
     * @access private
     * @var string
     */ 
    var $_fileName = ''; 

    /** 
     * image file
     * @access private
     * @var string
     */ 
    var $_file = '';

    /** 
     * subdirectory to create thumbnails in under the main directory 
     * @access private
     * @var string
     */ 
    var $_subDir = '';

    /** 
     * thumbnail main directory
     * @access private
     * @var string
     */ 
    var $_thumbDir = 'thumbs';

    /** 
     * image information
     * @access private
     * @var array
     */ 
    var $_imgInfo = array();

    /** 
     * thumbnail filename prefix
     * @access private
     * @var string
     */ 
    var $_prefix = 'th_';

    /** 
     * new image width
     * @access private
     * @var int
     */ 
    var $_newWidth = 0;

    /** 
     * new image height 
     * @access private
     * @var int
     */ 
    var $_newHeight = 0;

    /** 
     * jpeg quality
     * @access private
     * @var int
     */ 
    var $_quality = 90;

    /** 
     * default output directory will be same as image path
     * @access private
     * @var string
     */ 
    var $_outputDir = '';


    /**
     * constructor - sets original width/height
     *
     * @param string $file image file name
     * @param string $path path to image
     *
     * @param optional int $width
     * @param optional int $height
     * @param optional float $percentage of size change
     */
    function ImageSizer($file, $path, $width = 'auto', $height='auto', $percentage = 'auto') {

        // make sure filename is alphanumeric
        if (eregi('[^a-zA-Z0-9_]', $file)) {

            $this->_outputDir = $path;
            $this->_fileName = $path . DS . $file;
            $this->_file = $file;

            if ($this->_imgInfo = getimagesize($this->_fileName)) {

                $this->_setNewSize($width, $height, $percentage);

            } else {
                die($this->_file . ' is not a valid image file (jpeg, gif or png)');
            }

        } else {
            die($file . ' is not a valid filename.  Please only use alphanumeric and _ characters');
        }

    }//ImageSizer()


    /**
     * create thumbnail
     *
     * @return string path to thumbnail
     */
    function getThumbnail() {

        $destFile = ''; // full thumbnail path
        $cachedThumb = array();

        $imgPath = $this->_outputDir . $this->_thumbDir . '/';

        // check directories and create them if necessary
        if (!empty($this->_subDir)) {

            $imgPath = $this->_outputDir . $this->_thumbDir  . '/'. $this->_subDir . DS;

            if (!is_dir($imgPath)) {

                $this->_makeDir($this->_outputDir . $this->_thumbDir . '/', $this->_subDir);
            }

        } elseif (!is_dir($imgPath)) { 

            $this->_makeDir($this->_outputDir, $this->_thumbDir . '/');

        }


        $destFile = $imgPath . $this->_prefix . $this->_file;


        // if thumbnail already exists, resize if width & height don't match desired size
        if (is_file($destFile)) { 

            if ($cachedThumb = getimagesize($destFile)) {
                if (($cachedThumb[0] != $this->_newWidth) && ($cachedThumb[1] != $this->_newHeight)) {

                    if (unlink($destFile)) { 
                        $this->_resizeImage($destFile);
                    } else {
                        die('unable to delete current cached thumbnail - please check directory permisions');
                    }
                }

            }

        } else {
            $this->_resizeImage($destFile);
        }

        return $destFile;
        
    }//getThumbnail

    /**
     * set jpeg quality
     *
     * @param int $quality
     *
     * @return void
     */
    function setQuality($quality)     { $this->_quality = $quality; }

    /**
     * set sub directory to place thumbnails in (created under 'thumbs' dir)
     *
     * @param string $dir directory
     *
     * @return void
     */
    function setSubDir($dir)          { $this->_subDir = $dir; }

    /**
     * set thumbnail output directory - defaults to 'thumbs'
     *
     * @param string $outputDir 
     *
     * @return void
     */
    function setOutputDir($outputDir) { $this->_outputDir = $outputDir; }

    /**
     * set thumbnail main sub directory
     *
     * @param string $dir thumbnail sub directory name
     *
     * @return void
     */
    function setThumbDir($dir)        { $this->_thumbDir = $dir; }

    /**
     * set filename prefix for thumbnails
     *
     * @param string $prefix thumbnail prefix
     *
     * @return void
     */
    function setPrefix($prefix)       { $this->_prefix = $prefix; }
    

    /**
     * resize image
     * @access private
     *
     * @param string $destination path to save resized image to
     *
     * @return void
     */
    function _resizeImage($destination) {

        $currImage = $this->_openImage();

        if ($currImage) {

            // create the thumbnail
            $thumb = imagecreatetruecolor($this->_newWidth, $this->_newHeight);
            imagecopyresampled($thumb, $currImage, 0, 0, 0, 0, $this->_newWidth, $this->_newHeight, $this->_imgInfo[0], $this->_imgInfo[1]);

            switch ($this->_imgInfo[2]) {

                case IMAGETYPE_GIF:

                    imagegif($thumb, $destination);
                    break;

                case IMAGETYPE_JPEG:

                    imagejpeg($thumb, $destination, $this->_quality);
                    break;

                case IMAGETYPE_PNG:

                    imagepng($thumb, $destination);
                    break;
            }

            imagedestroy($thumb);
            imagedestroy($currImage);
        }

    }//_resizeImage()


    /**
     * opens image file - handles only common web formats (gif/jpg/png)
     * addtional formats can be added here
     * @access private
     *
     * @return resource image or FALSE on failure
     */
    function _openImage() {
 
        $img = FALSE;  // image resource
    
        switch ($this->_imgInfo[2]) { // image type flag
    
            case IMAGETYPE_GIF:
                $img = @imagecreatefromgif($this->_fileName);
                break;
    
            case IMAGETYPE_JPEG:
                $img = @imagecreatefromjpeg($this->_fileName);
                break;
    
            case IMAGETYPE_PNG:
                $img = @imagecreatefrompng($this->_fileName);
                break;
    
            default:
                $img = FALSE;
                break;
        }
        
        return $img;

    }//_openImage()


    /**
     * set new width and height 
     * calculates size based on parameter passed, using information from _imgInfo array
     * @access private
     *
     * @param int $width
     * @param int $height
     * @param float $percentage
     *
     * @return void
     */
    function _setNewSize($width = 'auto', $height = 'auto', $percentage = 'auto') {

        // make sure imgInfo contains at least 4 elements
        if (count($this->_imgInfo) >= 4) {

            if (!is_int($width) && is_int($height)) {

                // calculate new width based on height
                $this->_newWidth = floor($this->_imgInfo[0] * ($height / $this->_imgInfo[1]));
                $this->_newHeight = $height;

            } elseif (is_int($width) && !is_int($height)) {

                // calculate new height based on width
                $this->_newHeight = floor($this->_imgInfo[1] * ($width / $this->_imgInfo[0]));
                $this->_newWidth = $width;

            } elseif (!is_int($width) && !is_int($height) && is_float($percentage)) {

                // calculate width and height using percentage
                $this->_newWidth  = floor($this->_imgInfo[0] * $percentage);
                $this->_newHeight = floor($this->_imgInfo[1] * $percentage);

            }

        } else {
            die('missing original image information');
        }

    }//_setNewSize()


    /**
     * create directory
     * @access private
     *
     * @param string $root directory path to create in
     * @param string $dir name of directory to create
     * @param int $mode directory mode
     *
     * @return void
     */
    function _makeDir($root, $dir, $mode = 0777) {

        $old = umask(0);
        $dir = trim($dir, '/');
        $paths = explode('/', $dir);
        $gone = array();

        for ($i = 0; $i < count($paths); $i++) {
            $gone[] = $paths[$i];
            $d = $root . implode('/', $gone);
            if (!is_dir($d)) {
                mkdir($root . implode('/', $gone), $mode);
            }
        }

        umask($old);

    }//_makeDir()


}//Image class
?>
